//$Id: SprAbsCombiner.cc,v 1.3 2006/11/13 19:09:41 narsky Exp $

#include "StatPatternRecognition/SprExperiment.hh"
#include "StatPatternRecognition/SprAbsCombiner.hh"
#include "StatPatternRecognition/SprAbsTrainedClassifier.hh"
#include "StatPatternRecognition/SprAbsFilter.hh"
#include "StatPatternRecognition/SprData.hh"
#include "StatPatternRecognition/SprPoint.hh"
#include "StatPatternRecognition/SprEmptyFilter.hh"
#include "StatPatternRecognition/SprClass.hh"

#include <cassert>

using namespace std;


SprAbsCombiner::SprAbsCombiner(SprAbsFilter* data) 
  : 
  SprAbsClassifier(data),
  classifiers_(),
  classifierLabels_(),
  features_(0)
{}


SprAbsCombiner::SprAbsCombiner(SprAbsFilter* data, 
			       const std::vector<
			       const SprAbsTrainedClassifier*>& c,
			       const std::vector<std::string>& cLabels)
  :
  SprAbsClassifier(data),
  classifiers_(c),
  classifierLabels_(cLabels),
  features_(0)
{
  assert( !classifiers_.empty() );
  assert( classifiers_.size() == classifierLabels_.size() );
}


bool SprAbsCombiner::makeFeatures()
{
  // size
  int nClassifiers = classifiers_.size();
  if( nClassifiers == 0 ) return false;
  assert( nClassifiers == classifierLabels_.size() );

  // make data
  SprData* features = new SprData("features",classifierLabels_);
  vector<double> r(nClassifiers);
  for( int i=0;i<data_->size();i++ ) {
    const SprPoint* p = (*data_)[i];
    for( int j=0;j<nClassifiers;j++ )
      r[j] = classifiers_[j]->response(p);
    features->insert(p->class_,r);
  }

  // get weights
  vector<double> weights;
  data_->weights(weights);

  // get classes
  vector<SprClass> classes;
  data_->classes(classes);

  // make filter
  features_ = new SprEmptyFilter(features,classes,weights,true);

  // exit
  return true;
}
