//$Id: SprStdBackpropPlusAdaSplitApp.cc,v 1.5 2007/02/05 21:49:46 narsky Exp $

#include "StatPatternRecognition/SprExperiment.hh"
#include "StatPatternRecognition/SprStdBackprop.hh"
#include "StatPatternRecognition/SprTrainedStdBackprop.hh"
#include "StatPatternRecognition/SprAdaBoost.hh"
#include "StatPatternRecognition/SprTrainedAdaBoost.hh"
#include "StatPatternRecognition/SprAdaBoostCombiner.hh"
#include "StatPatternRecognition/SprTrainedAdaBoostCombiner.hh"
#include "StatPatternRecognition/SprClassifierReader.hh"
#include "StatPatternRecognition/SprBinarySplit.hh"
#include "StatPatternRecognition/SprData.hh"
#include "StatPatternRecognition/SprEmptyFilter.hh"
#include "StatPatternRecognition/SprAbsReader.hh"
#include "StatPatternRecognition/SprSimpleReader.hh"
#include "StatPatternRecognition/SprDataFeeder.hh"
#include "StatPatternRecognition/SprMyWriter.hh"
#include "StatPatternRecognition/SprTwoClassIDFraction.hh"

#include <stdlib.h>
#include <unistd.h>
#include <iostream>
#include <vector>
#include <string>
#include <memory>

using namespace std;


void cleanup(vector<const SprBinarySplit*>& splits)
{
  for( int i=0;i<splits.size();i++ )
    delete splits[i];
}


void help(const char* prog) 
{
  cout << "Usage:  " << prog 
       << " training_data_file " 
       << " net_configuration_file " 
       << " AdaBoostBinarySplit_configuration_file "<< endl;
  cout << "\t Options: " << endl;
  cout << "\t-h --- help                                        " << endl;
  cout << "\t-o output Tuple file                                 " << endl;
  cout << "\t-a input ascii file mode (see SprSimpleReader.hh) " << endl;
  cout << "\t-n number of AdaBoost training cycles              " << endl;
  cout << "\t-v verbose level (0=silent default,1,2)            " << endl;
  cout << "\t-f store trained AdaBoost to file                  " << endl;
  cout << "\t-r resume training for AdaBoost stored in file     " << endl;
  cout << "\t-t read validation/test data from a file           " << endl;
  cout << "\t\t (must be in same format as input data!!!        " << endl;
  cout << "\t-d frequency of print-outs for validation data     " << endl;
  cout << "\t-w scale all signal weights by this factor         " << endl;
}


int main(int argc, char ** argv)
{
  // check command line
  if( argc < 2 ) {
    help(argv[0]);
    return 1;
  }

  // init
  string hbkFile;
  int readMode = 1;
  unsigned cycles = 0;
  int verbose = 0;  
  string outFile;
  string resumeFile;
  string valFile;
  unsigned valPrint = 0;
  bool scaleWeights = false;
  double sW = 1.;

  // decode command line
  int c;
  extern char* optarg;
  while( (c = getopt(argc,argv,"ho:a:n:v:f:r:t:d:w:")) != EOF ) {
    switch( c )
      {
      case 'h' :
	help(argv[0]);
	return 1;
      case 'o' :
	hbkFile = optarg;
	break;
      case 'a' :
	readMode = (optarg==0 ? 1 : atoi(optarg));
	break;
      case 'n' :
	cycles = (optarg==0 ? 1 : atoi(optarg));
	break;
      case 'v' :
	verbose = (optarg==0 ? 0 : atoi(optarg));
	break;
	break;
      case 'f' :
	outFile = optarg;
	break;
      case 'r' :
	resumeFile = optarg;
	break;
      case 't' :
	valFile = optarg;
	break;
      case 'd' :
	valPrint = (optarg==0 ? 0 : atoi(optarg));
	break;
      case 'w' :
	if( optarg != 0 ) {
	  scaleWeights = true;
	  sW = atof(optarg);
	}
	break;
      }
  }

  // There have to be 3 arguments after all options.
  string trFile = argv[argc-3];
  string netFile = argv[argc-2];
  string adaFile = argv[argc-1];
  if( trFile.empty() ) {
    cerr << "No training file is specified." << endl;
    return 1;
  }
  if( netFile.empty() ) {
    cerr << "No net file is specified." << endl;
    return 1;
  }
  if( adaFile.empty() ) {
    cerr << "No AdaBoost file is specified." << endl;
    return 1;
  }

  // read training data from file
  SprSimpleReader reader(readMode);
  auto_ptr<SprAbsFilter> filter(reader.read(trFile.c_str()));
  if( filter.get() == 0 ) {
    cerr << "Unable to read data from file " << trFile.c_str() << endl;
    return 2;
  }
  vector<string> vars;
  filter->vars(vars);
  cout << "Read data from file " << trFile.c_str() 
       << " for variables";
  for( int i=0;i<vars.size();i++ ) 
    cout << " \"" << vars[i].c_str() << "\"";
  cout << endl;
  cout << "Total number of points read: " << filter->size() << endl;
  cout << "Points in class 0: " << filter->ptsInClass(0)
       << " 1: " << filter->ptsInClass(1) << endl;

  // scale weights
  if( scaleWeights ) {
    cout << "Signal weights are multiplied by " << sW << endl;
    filter->scaleWeights(1,sW);
  }

  // read validation data from file
  auto_ptr<SprAbsFilter> valFilter;
  if( !valFile.empty() && valPrint!=0 ) {
    SprSimpleReader valReader(readMode);
    valFilter.reset(valReader.read(valFile.c_str()));
    if( valFilter.get() == 0 ) {
      cerr << "Unable to read data from file " << valFile.c_str() << endl;
      return 2;
    }
    vector<string> valVars;
    valFilter->vars(valVars);
    cout << "Read validation data from file " << valFile.c_str() 
	 << " for variables";
    for( int i=0;i<valVars.size();i++ ) 
      cout << " \"" << valVars[i].c_str() << "\"";
    cout << endl;
    cout << "Total number of points read: " << valFilter->size() << endl;
    cout << "Points in class 0: " << valFilter->ptsInClass(0)
	 << " 1: " << valFilter->ptsInClass(1) << endl;
  }

  // scale weights
  if( scaleWeights && valFilter.get()!=0 )
    valFilter->scaleWeights(1,sW);

  // read net
  SprStdBackprop readNet(filter.get());
  if( !readNet.readSNNS(netFile.c_str()) ) {
    cerr << "Unable to read net file " << netFile.c_str() << endl;
    return 3;
  }
  else {
    cout << "Read net configuration file:" << endl;
    readNet.print(cout);
  }
  auto_ptr<SprTrainedStdBackprop> net(readNet.makeTrained());

  // read AdaBoost
  auto_ptr<SprTrainedAdaBoost> trainedAda;
  if( !adaFile.empty() ) {
    trainedAda.reset(SprClassifierReader::readTrained<
		     SprTrainedAdaBoost>(adaFile.c_str(),"AdaBoost",verbose));
    if( trainedAda.get() == 0 ) {
      cerr << "Failed to read saved AdaBoost from file " 
	   << adaFile.c_str() << endl;
      return 4;
    }
    cout << "Read saved AdaBoost from file " << adaFile.c_str() << endl;
  }

  // prepare AdaBoost combiner
  cycles *= 2;// since we are combining two classifiers
  SprAdaBoostCombiner abCombiner(filter.get(),cycles);
  abCombiner.addClassifier(net.get(),"stdnn");
  abCombiner.addClassifier(trainedAda.get(),"ada");
  if( !abCombiner.closeClassifierList() ) {
    cerr << "Unable to close classifier list for AdaBoost combiner." << endl;
    return 5;
  }
  SprAbsFilter* features = abCombiner.features();

  // set up binary splits
  SprTwoClassIDFraction crit;
  vector<const SprBinarySplit*> splits;
  for( int i=0;i<features->dim();i++ ) {
    SprBinarySplit* s = new SprBinarySplit(features,&crit,i);
    splits.push_back(s);
    if( !abCombiner.addTrainable(s,SprUtils::lowerBound(0.5)) ) {
      cerr << "Unable to add binary split to AdaBoost combiner." << endl;
      cleanup(splits);
      return 6;
    }
  }

  // read saved combiner from file
  if( !resumeFile.empty() ) {
    if( !SprClassifierReader::readTrainable(resumeFile.c_str(),
					    abCombiner.ada(),
					    verbose) ) {
      cerr << "Failed to read saved AdaBoost combiner from file " 
	   << resumeFile.c_str() << endl;
      cleanup(splits);
      return 7;
    }
    cout << "Read saved AdaBoost from file " << resumeFile.c_str()
	 << " with " << abCombiner.nTrained() 
	 << " trained classifiers." << endl;
  }

  // set validation
  if( valFilter.get()!=0 && !valFilter->empty() )
    abCombiner.setValidation(valFilter.get(),valPrint);

  // train
  if( !abCombiner.train(verbose) ) {
    cerr << "Unable to train AdaBoost combiner." << endl;
    cleanup(splits);
    return 8;
  }

  // save trained combiner
  if( !outFile.empty() ) {
    if( !abCombiner.store(outFile.c_str()) ) {
      cerr << "Cannot store AdaBoost combiner in file " 
	   << outFile.c_str() << endl;
      cleanup(splits);
      return 9;
    }
  }

  // make trained combiner
  auto_ptr<SprTrainedAdaBoostCombiner> 
    trainedCombiner(abCombiner.makeTrained());

  // make histogram if requested
  if( hbkFile.empty() ) {
    cleanup(splits);
    return 0;
  }

  // make a writer
  SprMyWriter hbk("training");
  if( !hbk.init(hbkFile.c_str()) ) {
    cerr << "Unable to open output file " << hbkFile.c_str() << endl;
    cleanup(splits);
    return 10;
  }

  // feed
  SprDataFeeder feeder(filter.get(),&hbk);
  feeder.addClassifier(net.get(),"stdnn");
  feeder.addClassifier(trainedAda.get(),"ada");
  feeder.addClassifier(trainedCombiner.get(),"comb");
  if( !feeder.feed(1000) ) {
    cerr << "Cannot feed data into file " << hbkFile.c_str() << endl;
    cleanup(splits);
    return 11;
  }

  // clean up
  cleanup(splits);

  // exit
  return 0;
}
